@@ -107,6 +107,11 @@ def _create_sg_rules_with_container_ports(matched_pods, container_ports,
     for pod, container_port in container_ports:
         pod_namespace = pod['metadata']['namespace']
         pod_ip = driver_utils.get_pod_ip(pod)
+        if not pod_ip:
+            LOG.debug("Skipping SG rule creation for pod %s due to "
+                      "no IP assigned", pod['metadata']['name'])
+            continue
+
         pod_info = {pod_ip: pod_namespace}
         matched = True
         if allow_all or namespace:
@@ -120,6 +125,10 @@ def _create_sg_rules_with_container_ports(matched_pods, container_ports,
                     matched_pods[container_port] = pod_info
         else:
             pod_ip = driver_utils.get_pod_ip(rule_selected_pod)
+            if not pod_ip:
+                LOG.debug("Skipping SG rule creation for pod %s due to no IP "
+                          "assigned", rule_selected_pod['metadata']['name'])
+                continue
             sg_rule = driver_utils.create_security_group_rule_body(
                 sg_id, direction, container_port,
                 protocol=port.get('protocol'),
@@ -188,6 +197,10 @@ def _create_sg_rules(crd, pod, pod_selector, rule_block,
                      allow_all=False):
     pod_labels = pod['metadata'].get('labels')
     pod_ip = driver_utils.get_pod_ip(pod)
+    if not pod_ip:
+        LOG.debug("Skipping SG rule creation for pod %s due to "
+                  "no IP assigned", pod['metadata']['name'])
+        return None
 
     # NOTE (maysams) No need to differentiate between podSelector
     # with empty value or with '{}', as they have same result in here.
@@ -274,18 +287,28 @@ def _parse_selectors_on_namespace(crd, direction, pod_selector,
                         matched = True
                         for pod in pods:
                             pod_ip = driver_utils.get_pod_ip(pod)
+                            if not pod_ip:
+                                pod_name = pod['metadata']['name']
+                                LOG.debug("Skipping SG rule creation for pod "
+                                          "%s due to no IP assigned", pod_name)
+                                continue
                             crd_rules.append(_create_sg_rule(
                                 sg_id, direction, pod_ip, port=port,
                                 namespace=ns_name))
             else:
                 for pod in pods:
                     pod_ip = driver_utils.get_pod_ip(pod)
+                    if not pod_ip:
+                        pod_name = pod['metadata']['name']
+                        LOG.debug("Skipping SG rule creation for pod %s due"
+                                  " to no IP assigned", pod_name)
+                        continue
                     matched = True
                     crd_rules.append(_create_sg_rule(
                         sg_id, direction, pod_ip,
                         namespace=ns_name))
         else:
-            ns_pods = driver_utils.get_pods(ns_selector)
+            ns_pods = driver_utils.get_pods(ns_selector)['items']
             ns_cidr = driver_utils.get_namespace_subnet_cidr(namespace)
             if 'ports' in rule_block:
                 for port in rule_block['ports']:
@@ -353,12 +376,12 @@ def _parse_rules_on_delete_namespace(rule_list, direction, ns_name):
         LOG.debug('Parsing %(dir)s Rule %(r)s', {'dir': direction,
                                                  'r': rule})
         rule_namespace = rule.get('namespace', None)
-        remote_ip_prefixes = rule.get('remote_ip_prefixes', [])
+        remote_ip_prefixes = rule.get('remote_ip_prefixes', {})
         if rule_namespace and rule_namespace == ns_name:
             matched = True
             driver_utils.delete_security_group_rule(
                 rule['security_group_rule']['id'])
-        for remote_ip, namespace in remote_ip_prefixes:
+        for remote_ip, namespace in list(remote_ip_prefixes.items()):
             if namespace == ns_name:
                 matched = True
                 remote_ip_prefixes.pop(remote_ip)
@@ -378,7 +401,7 @@ def _parse_rules_on_delete_pod(rule_list, direction, pod_ip):
                                                  'r': rule})
         remote_ip_prefix = rule['security_group_rule'].get(
             'remote_ip_prefix')
-        remote_ip_prefixes = rule.get('remote_ip_prefixes', [])
+        remote_ip_prefixes = rule.get('remote_ip_prefixes', {})
         if remote_ip_prefix and remote_ip_prefix == pod_ip:
             matched = True
             driver_utils.delete_security_group_rule(
@@ -445,12 +468,17 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
                 driver_utils.patch_kuryrnetworkpolicy_crd(crd, i_rules,
                                                           e_rules,
                                                           crd_selector)
+            if i_matched:
                 crd_pod_selectors.append(crd_selector)
         return crd_pod_selectors
 
     def delete_sg_rules(self, pod):
         LOG.debug("Deleting sg rule for pod: %s", pod['metadata']['name'])
         pod_ip = driver_utils.get_pod_ip(pod)
+        if not pod_ip:
+            LOG.debug("Skipping SG rule deletion as pod %s has no IP assigned",
+                      pod['metadata']['name'])
+            return None
         crd_pod_selectors = []
         knp_crds = driver_utils.get_kuryrnetpolicy_crds()
         for crd in knp_crds.get('items'):
@@ -467,6 +495,7 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
                 driver_utils.patch_kuryrnetworkpolicy_crd(crd, i_rules,
                                                           e_rules,
                                                           crd_selector)
+            if i_matched:
                 crd_pod_selectors.append(crd_selector)
         return crd_pod_selectors
 
@@ -482,6 +511,7 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
         LOG.debug("Deleting sg rule for namespace: %s",
                   ns_name)
 
+        crd_selectors = []
         knp_crds = driver_utils.get_kuryrnetpolicy_crds()
         for crd in knp_crds.get('items'):
             crd_selector = crd['spec'].get('podSelector')
@@ -496,13 +526,14 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
             if i_matched or e_matched:
                 driver_utils.patch_kuryrnetworkpolicy_crd(
                     crd, i_rules, e_rules, crd_selector)
+            if i_matched:
+                crd_selectors.append(crd_selector)
+        return crd_selectors
 
     def create_namespace_sg_rules(self, namespace):
-        kubernetes = clients.get_kubernetes_client()
         ns_name = namespace['metadata']['name']
         LOG.debug("Creating sg rule for namespace: %s", ns_name)
-        namespace = kubernetes.get(
-            '{}/namespaces/{}'.format(constants.K8S_API_BASE, ns_name))
+        crd_selectors = []
         knp_crds = driver_utils.get_kuryrnetpolicy_crds()
         for crd in knp_crds.get('items'):
             crd_selector = crd['spec'].get('podSelector')
@@ -516,12 +547,17 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
                 driver_utils.patch_kuryrnetworkpolicy_crd(crd, i_rules,
                                                           e_rules,
                                                           crd_selector)
+            if i_matched:
+                crd_selectors.append(crd_selector)
+        return crd_selectors
 
     def update_namespace_sg_rules(self, namespace):
         LOG.debug("Updating sg rule for namespace: %s",
                   namespace['metadata']['name'])
-        self.delete_namespace_sg_rules(namespace)
-        self.create_namespace_sg_rules(namespace)
+        crd_selectors = []
+        crd_selectors.extend(self.delete_namespace_sg_rules(namespace))
+        crd_selectors.extend(self.create_namespace_sg_rules(namespace))
+        return crd_selectors
 
     def create_namespace_sg(self, namespace, project_id, crd_spec):
         LOG.debug("Security group driver does not create SGs for the "
