@@ -122,6 +122,9 @@ class NoopVIFPool(base.VIFPoolDriver):
     def update_vif_sgs(self, pod, sgs):
         self._drv_vif.update_vif_sgs(pod, sgs)
 
+    def remove_sg_from_pools(self, sg_id, net_id):
+        pass
+
     def sync_pools(self):
         pass
 
@@ -157,6 +160,7 @@ class BaseVIFPool(base.VIFPoolDriver):
     def __init__(self):
         # Note(ltomasbo) Execute the port recycling periodic actions in a
         # background thread
+        self._recovered_pools = False
         eventlet.spawn(self._return_ports_to_pool)
 
     def set_vif_driver(self, driver):
@@ -212,7 +216,11 @@ class BaseVIFPool(base.VIFPoolDriver):
         # REVISIT(ltomasbo): Drop the subnets parameter and get the information
         # from the pool_key, which will be required when multi-network is
         # supported
+        if not self._recovered_pools:
+            LOG.info("Kuryr-controller not yet ready to populate pools.")
+            raise exceptions.ResourceNotReady(pod)
         now = time.time()
+        last_update = 0
         pool_updates = self._last_update.get(pool_key)
         if pool_updates:
             last_update = pool_updates.get(security_groups, 0)
@@ -241,9 +249,13 @@ class BaseVIFPool(base.VIFPoolDriver):
                 self._available_ports_pools.setdefault(
                     pool_key, {}).setdefault(
                         security_groups, []).append(vif.id)
+            if not vifs:
+                self._last_update[pool_key] = {security_groups: last_update}
 
-    def release_vif(self, pod, vif, project_id, security_groups):
-        host_addr = self._get_host_addr(pod)
+    def release_vif(self, pod, vif, project_id, security_groups,
+                    host_addr=None):
+        if not host_addr:
+            host_addr = self._get_host_addr(pod)
 
         pool_key = self._get_pool_key(host_addr, project_id, vif.network.id,
                                       None)
@@ -288,6 +300,39 @@ class BaseVIFPool(base.VIFPoolDriver):
     def delete_network_pools(self, net_id):
         raise NotImplementedError()
 
+    def remove_sg_from_pools(self, sg_id, net_id):
+        neutron = clients.get_neutron_client()
+        for pool_key, pool_ports in list(self._available_ports_pools.items()):
+            if self._get_pool_key_net(pool_key) != net_id:
+                continue
+            for sg_key, ports in list(pool_ports.items()):
+                if sg_id not in sg_key:
+                    continue
+                # remove the pool associated to that SG
+                try:
+                    del self._available_ports_pools[pool_key][sg_key]
+                except KeyError:
+                    LOG.debug("SG already removed from the pool. Ports "
+                              "already re-used, no need to change their "
+                              "associated SGs.")
+                    continue
+                for port_id in ports:
+                    # remove all SGs from the port to be reused
+                    neutron.update_port(
+                        port_id,
+                        {
+                            "port": {
+                                'security_groups': []
+                            }
+                        })
+                    # add the port to the default pool
+                    self._available_ports_pools[pool_key].setdefault(
+                        tuple([]), []).append(port_id)
+                # NOTE(ltomasbo): as this ports were not created for this
+                # pool, ensuring they are used first, marking them as the
+                # most outdated
+                self._last_update[pool_key] = {tuple([]): 0}
+
     def _create_healthcheck_file(self):
         # Note(ltomasbo): Create a health check file when the pre-created
         # ports are loaded into their corresponding pools. This file is used
@@ -314,10 +359,6 @@ class BaseVIFPool(base.VIFPoolDriver):
         self._existing_vifs = collections.defaultdict()
         self._recyclable_ports = collections.defaultdict()
         self._last_update = collections.defaultdict()
-        # NOTE(ltomasbo): Ensure previously created ports are recovered into
-        # their respective pools
-        self._cleanup_leftover_ports()
-        self._recover_precreated_ports()
 
     def _get_trunks_info(self):
         """Returns information about trunks and their subports.
@@ -443,7 +484,7 @@ class NeutronVIFPool(BaseVIFPool):
             pool_updates = self._last_update.get(pool_key, {})
             if not pool_updates:
                 # No pools update info. Selecting a random one
-                for sg_group, ports in pool_ports.items():
+                for sg_group, ports in list(pool_ports.items()):
                     if len(ports) > 0:
                         port_id = pool_ports[sg_group].pop()
                         break
@@ -451,7 +492,7 @@ class NeutronVIFPool(BaseVIFPool):
                     raise exceptions.ResourceNotReady(pod)
             else:
                 min_date = -1
-                for sg_group, date in pool_updates.items():
+                for sg_group, date in list(pool_updates.items()):
                     if pool_ports.get(sg_group):
                         if min_date == -1 or date < min_date:
                             min_date = date
@@ -519,7 +560,7 @@ class NeutronVIFPool(BaseVIFPool):
                     sg_current[port['id']] = tuple(sorted(
                         port['security_groups']))
 
-        for port_id, pool_key in self._recyclable_ports.copy().items():
+        for port_id, pool_key in list(self._recyclable_ports.items()):
             if (not oslo_cfg.CONF.vif_pool.ports_pool_max or
                 self._get_pool_size(pool_key) <
                     oslo_cfg.CONF.vif_pool.ports_pool_max):
@@ -558,6 +599,14 @@ class NeutronVIFPool(BaseVIFPool):
             except KeyError:
                 LOG.debug('Port already recycled: %s', port_id)
 
+    def sync_pools(self):
+        super(NeutronVIFPool, self).sync_pools()
+        # NOTE(ltomasbo): Ensure previously created ports are recovered into
+        # their respective pools
+        self._cleanup_leftover_ports()
+        self._recover_precreated_ports()
+        self._recovered_pools = True
+
     def _recover_precreated_ports(self):
         attrs = {'device_owner': kl_const.DEVICE_OWNER}
         tags = config.CONF.neutron_defaults.resource_tags
@@ -607,7 +656,7 @@ class NeutronVIFPool(BaseVIFPool):
         self._create_healthcheck_file()
 
     def delete_network_pools(self, net_id):
-        if not hasattr(self, '_available_ports_pools'):
+        if not self._recovered_pools:
             LOG.info("Kuryr-controller not yet ready to delete network "
                      "pools.")
             raise exceptions.ResourceNotReady(net_id)
@@ -618,7 +667,7 @@ class NeutronVIFPool(BaseVIFPool):
         # on the available_ports_pools dict. The next call forces it to be on
         # that dict before cleaning it up
         self._trigger_return_to_pool()
-        for pool_key, ports in self._available_ports_pools.items():
+        for pool_key, ports in list(self._available_ports_pools.items()):
             if self._get_pool_key_net(pool_key) != net_id:
                 continue
             ports_id = []
@@ -657,6 +706,41 @@ class NestedVIFPool(BaseVIFPool):
     def set_vif_driver(self, driver):
         self._drv_vif = driver
 
+    def _get_parent_port_id(self, vif):
+        neutron = clients.get_neutron_client()
+        args = {}
+        if config.CONF.neutron_defaults.resource_tags:
+            args['tags'] = config.CONF.neutron_defaults.resource_tags
+        trunks = neutron.list_trunks(**args)
+        for trunk in trunks['trunks']:
+            for sp in trunk['sub_ports']:
+                if sp['port_id'] == vif.id:
+                    return trunk['port_id']
+
+        return None
+
+    def release_vif(self, pod, vif, project_id, security_groups):
+        try:
+            host_addr = self._get_host_addr(pod)
+        except KeyError:
+            name = pod['metadata']['name']
+            LOG.warning("Pod %s does not have status.hostIP field set when "
+                        "getting deleted. This is unusual. Trying to "
+                        "determine the IP by calling Neutron.",
+                        name)
+
+            parent_id = self._get_parent_port_id(vif)
+            if not parent_id:
+                LOG.warning("Port %s not found, ignoring its release request.",
+                            vif.id)
+                return
+
+            host_addr = self._get_parent_port_ip(parent_id)
+            LOG.debug("Determined hostIP for pod %s is %s", name, host_addr)
+
+        super(NestedVIFPool, self).release_vif(
+            pod, vif, project_id, security_groups, host_addr=host_addr)
+
     def _get_port_from_pool(self, pool_key, pod, subnets, security_groups):
         try:
             pool_ports = self._available_ports_pools[pool_key]
@@ -671,7 +755,7 @@ class NestedVIFPool(BaseVIFPool):
             pool_updates = self._last_update.get(pool_key, {})
             if not pool_updates:
                 # No pools update info. Selecting a random one
-                for sg_group, ports in pool_ports.items():
+                for sg_group, ports in list(pool_ports.items()):
                     if len(ports) > 0:
                         port_id = pool_ports[sg_group].pop()
                         break
@@ -679,7 +763,7 @@ class NestedVIFPool(BaseVIFPool):
                     raise exceptions.ResourceNotReady(pod)
             else:
                 min_date = -1
-                for sg_group, date in pool_updates.items():
+                for sg_group, date in list(pool_updates.items()):
                     if pool_ports.get(sg_group):
                         if min_date == -1 or date < min_date:
                             min_date = date
@@ -746,7 +830,7 @@ class NestedVIFPool(BaseVIFPool):
                     sg_current[subport['id']] = tuple(sorted(
                         subport['security_groups']))
 
-        for port_id, pool_key in self._recyclable_ports.copy().items():
+        for port_id, pool_key in list(self._recyclable_ports.items()):
             if (not oslo_cfg.CONF.vif_pool.ports_pool_max or
                 self._get_pool_size(pool_key) <
                     oslo_cfg.CONF.vif_pool.ports_pool_max):
@@ -806,6 +890,14 @@ class NestedVIFPool(BaseVIFPool):
         parent_port = neutron.show_port(port_id).get('port')
         return parent_port['fixed_ips'][0]['ip_address']
 
+    def sync_pools(self):
+        super(NestedVIFPool, self).sync_pools()
+        # NOTE(ltomasbo): Ensure previously created ports are recovered into
+        # their respective pools
+        self._recover_precreated_ports()
+        self._recovered_pools = True
+        eventlet.spawn(self._cleanup_leftover_ports)
+
     def _recover_precreated_ports(self):
         self._precreated_ports(action='recover')
         LOG.info("PORTS POOL: pools updated with pre-created ports")
@@ -886,7 +978,7 @@ class NestedVIFPool(BaseVIFPool):
 
     @lockutils.synchronized('return_to_pool_nested')
     def populate_pool(self, trunk_ip, project_id, subnets, security_groups):
-        if not hasattr(self, '_available_ports_pools'):
+        if not self._recovered_pools:
             LOG.info("Kuryr-controller not yet ready to populate pools.")
             raise exceptions.ResourceNotReady(trunk_ip)
         pool_key = self._get_pool_key(trunk_ip, project_id, None, subnets)
@@ -934,7 +1026,7 @@ class NestedVIFPool(BaseVIFPool):
         self._remove_precreated_ports(trunk_ips)
 
     def delete_network_pools(self, net_id):
-        if not hasattr(self, '_available_ports_pools'):
+        if not self._recovered_pools:
             LOG.info("Kuryr-controller not yet ready to delete network "
                      "pools.")
             raise exceptions.ResourceNotReady(net_id)
@@ -944,7 +1036,7 @@ class NestedVIFPool(BaseVIFPool):
         # on the available_ports_pools dict. The next call forces it to be on
         # that dict before cleaning it up
         self._trigger_return_to_pool()
-        for pool_key, ports in self._available_ports_pools.items():
+        for pool_key, ports in list(self._available_ports_pools.items()):
             if self._get_pool_key_net(pool_key) != net_id:
                 continue
             trunk_id = self._get_trunk_id(neutron, pool_key)
@@ -1025,6 +1117,12 @@ class MultiVIFPool(base.VIFPoolDriver):
         pod_vif_type = self._get_pod_vif_type(pod)
         self._vif_drvs[pod_vif_type].update_vif_sgs(pod, sgs)
 
+    def remove_sg_from_pools(self, sg_id, net_id):
+        for vif_drv in self._vif_drvs.values():
+            if str(vif_drv) == 'NoopVIFPool':
+                continue
+            vif_drv.remove_sg_from_pools(sg_id, net_id)
+
     def delete_network_pools(self, net_id):
         for vif_drv in self._vif_drvs.values():
             if str(vif_drv) == 'NoopVIFPool':
