@@ -15,6 +15,7 @@
 import contextlib
 import itertools
 import os
+import ssl
 
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -22,8 +23,10 @@ import requests
 
 from kuryr.lib._i18n import _
 from kuryr_kubernetes import config
+from kuryr_kubernetes import constants
 from kuryr_kubernetes import exceptions as exc
 
+CONF = config.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -125,6 +128,38 @@ class K8sClient(object):
             return response.json().get('status')
         raise exc.K8sClientException(response.text)
 
+    def patch_node_annotations(self, node, annotation_name, value):
+        content_type = 'application/json-patch+json'
+        path = '{}/nodes/{}/'.format(constants.K8S_API_BASE, node)
+        value = jsonutils.dumps(value)
+        url, header = self._get_url_and_header(path, content_type)
+
+        data = [{'op': 'add',
+                 'path': '/metadata/annotations/{}'.format(annotation_name),
+                 'value': value}]
+
+        response = requests.patch(url, data=jsonutils.dumps(data),
+                                  headers=header, cert=self.cert,
+                                  verify=self.verify_server)
+        if response.ok:
+            return response.json().get('status')
+        raise exc.K8sClientException(response.text)
+
+    def remove_node_annotations(self, node, annotation_name):
+        content_type = 'application/json-patch+json'
+        path = '{}/nodes/{}/'.format(constants.K8S_API_BASE, node)
+        url, header = self._get_url_and_header(path, content_type)
+
+        data = [{'op': 'remove',
+                 'path': '/metadata/annotations/{}'.format(annotation_name)}]
+
+        response = requests.patch(url, data=jsonutils.dumps(data),
+                                  headers=header, cert=self.cert,
+                                  verify=self.verify_server)
+        if response.ok:
+            return response.json().get('status')
+        raise exc.K8sClientException(response.text)
+
     def post(self, path, body):
         LOG.debug("Post %(path)s: %(body)s", {'path': path, 'body': body})
         url = self._base_url + path
@@ -185,15 +220,16 @@ class K8sClient(object):
                     'annotations', {})
 
                 for k, v in annotations.items():
-                    if v != retrieved_annotations.get(k, v):
+                    if v != retrieved_annotations.get(k):
                         break
                 else:
-                    # No conflicting annotations found. Retry patching
-                    resource_version = new_version
-                    continue
-                LOG.debug("Annotations for %(path)s already present: "
-                          "%(names)s", {'path': path,
-                                        'names': retrieved_annotations})
+                    LOG.debug("Annotations for %(path)s already present: "
+                              "%(names)s", {'path': path,
+                                            'names': retrieved_annotations})
+                    return retrieved_annotations
+                # Retry patching with updated resourceVersion
+                resource_version = new_version
+                continue
 
             LOG.error("Exception response, headers: %(headers)s, "
                       "content: %(content)s, text: %(text)s"
@@ -206,21 +242,48 @@ class K8sClient(object):
                 raise exc.K8sClientException(response.text)
 
     def watch(self, path):
-        params = {'watch': 'true'}
         url = self._base_url + path
+        resource_version = None
         header = {}
+        timeouts = (CONF.kubernetes.watch_connection_timeout,
+                    CONF.kubernetes.watch_read_timeout)
         if self.token:
             header.update({'Authorization': 'Bearer %s' % self.token})
 
-        # TODO(ivc): handle connection errors and retry on failure
         while True:
-            with contextlib.closing(
-                    requests.get(url, params=params, stream=True,
-                                 cert=self.cert, verify=self.verify_server,
-                                 headers=header)) as response:
-                if not response.ok:
-                    raise exc.K8sClientException(response.text)
-                for line in response.iter_lines():
-                    line = line.decode('utf-8').strip()
-                    if line:
-                        yield jsonutils.loads(line)
+            try:
+                params = {'watch': 'true'}
+                if resource_version:
+                    params['resourceVersion'] = resource_version
+                with contextlib.closing(
+                        requests.get(
+                            url, params=params, stream=True, cert=self.cert,
+                            verify=self.verify_server, headers=header,
+                            timeout=timeouts)) as response:
+                    if not response.ok:
+                        raise exc.K8sClientException(response.text)
+                    for line in response.iter_lines():
+                        line = line.decode('utf-8').strip()
+                        if line:
+                            line_dict = jsonutils.loads(line)
+                            yield line_dict
+                            # Saving the resourceVersion in case of a restart.
+                            # At this point it's safely passed to handler.
+                            m = line_dict.get('object', {}).get('metadata', {})
+                            resource_version = m.get('resourceVersion', None)
+            except (requests.ReadTimeout, ssl.SSLError) as e:
+                if isinstance(e, ssl.SSLError) and e.args != ('timed out',):
+                    raise
+
+                LOG.warning('%ds without data received from watching %s. '
+                            'Retrying the connection with resourceVersion=%s.',
+                            timeouts[1], path, params.get('resourceVersion'))
+            except requests.exceptions.ChunkedEncodingError:
+                LOG.warning("Connection to %s closed when watching. This "
+                            "mostly happens when Octavia's Amphora closes "
+                            "connection due to lack of activity for 50s. "
+                            "Since Rocky Octavia this is configurable and "
+                            "should be set to at least 20m, so check timeouts "
+                            "on Kubernetes API LB listener. Restarting "
+                            "connection with resourceVersion=%s.", path,
+                            params.get('resourceVersion'))
